program Brill

  ! Equations taken from...
  !
  ! 1967 Br. J. Appl. Phys. v18 1415 
  ! 
  !  Tables of the Brillouin function and of the related function for the
  !  spontaneous magnetization
  !
  !  M. I. DARBY
  !
  ! If you use the results of this little program, please reference that paper.

  implicit none

  real(kind=4) :: J,M,v1,v2,v3,delta,x,deltamin,Mmin,B_J
  real(kind=4) :: red_temps(17),T,thresh
  integer :: i,k,npts
  character*32 :: outname

  red_temps(1) = 0.0
  red_temps(2) = 0.1
  red_temps(3) = 0.2
  red_temps(4) = 0.3
  red_temps(5) = 0.4
  red_temps(6) = 0.45
  red_temps(7) = 0.50
  red_temps(8) = 0.55
  red_temps(9) = 0.60
  red_temps(10) = 0.65
  red_temps(11) = 0.70
  red_temps(12) = 0.75
  red_temps(13) = 0.80
  red_temps(14) = 0.85
  red_temps(15) = 0.90
  red_temps(16) = 0.95
  red_temps(17) = 1.0

  write(6,*)'What is J?'
  read(5,*) J
  npts = 1000000
  thresh = 1.0/real(npts)
  

  write(outname,'(a16,f3.1)')'Brillouin_curve.',J

  write(6,*)trim(outname)

  v1 = (2.0*J+1)/(2.0*J)
  v2 = (3.0*J) / (J+1.0)
  v3 = 1.0/ (2.0*J)

  ! Loop over T/Tc values, finding where M/Mo gives the
  ! requisite solution

  open(unit=1,file=outname)

  write(1,*)'#   T/Tc    M/M(0)    delta   J=',J
  write(6,*)'#   T/Tc    M/M(0)    delta   J=',J

  write(1,*)0.0,1.0
  write(6,*)0.0,1.0
  do i = 2,16
     T =  red_temps(i)
     ! just loop over a grid...
     deltamin = 10000.0
     Mmin = 0.0
     do k = npts,0,-1
        M = real(k)/real(npts)
        x = v2 * M / T
        B_J = v1*cosh(v1*x)/sinh(v1*x) - v3*cosh(v3*x)/sinh(v3*x)
        delta = abs(B_J - T / v2 * x)
        if (delta.lt.deltamin) then
           deltamin = delta
           Mmin = M
        end if
        if(deltamin.lt.thresh)exit
     end do !k
     write(1,*)T,Mmin,deltamin
     write(6,*)T,Mmin,deltamin
  end do ! i loop over array
  write(1,*)1.0,0.0
  write(6,*)1.0,0.0

  close(1)

  stop

end program Brill

